﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace chkssl
{
        using System;
        using System.Net;
        using System.Net.Security;
        using System.Security.Cryptography.X509Certificates;

        class Program
        {
            private static string opt_url = "";
            private static int opt_port = 443;
            private static bool opt_showCertDetails = false;
            private static bool opt_showChain = false;
            private static bool opt_showRaw = false;

            static void Main(string[] args)
            {
                ServicePointManager.ServerCertificateValidationCallback += ServerCertificateValidationCallback;

                // ghetto arg parsing

                for (int i=0; i < args.Count(); i++) {
                    opt_url = args[i];    // this will update every time

                    if ((args[i] == "-p") || (args[i] == "--port") || (args[i] == "/p") || (args[i] == "/port")) {
                        try {
                            opt_port = Convert.ToInt32(args[i+1]);
                        }
                        catch {
                            Console.WriteLine("Invalid port specification, aborting...");
                            Environment.Exit(2);
                        }
                    }

                    if ((args[i] == "-d") || (args[i] == "--details") || (args[i] == "/d") || (args[i] == "/details"))
                    {
                        opt_showCertDetails = true;
                    }

                    if ((args[i] == "-c") || (args[i] == "--chain") || (args[i] == "/c") || (args[i] == "/chain"))
                    {
                        opt_showChain = true;
                    }

                    if ((args[i] == "-r") || (args[i] == "--raw") || (args[i] == "/r") || (args[i] == "/raw"))
                    {
                        opt_showRaw = true;
                    }

                    if ((args[i] == "-a") || (args[i] == "--all") || (args[i] == "/a") || (args[i] == "/all"))
                    {
                        
                        opt_showChain = true;
                        opt_showCertDetails = true;
                    }

                    if ((args[i] == "-h") || (args[i] == "--help") || (args[i] == "/h") || (args[i] == "/help") || (args[i] == "/?") || (args[i] == "-?"))
                    {
                        ShowHelp();
                        Environment.Exit(0);
                    }

                    if ((args[i] == "-v") || (args[i] == "--version") || (args[i] == "/v") || (args[i] == "/version"))
                    {
                        ShowVersion();
                        Environment.Exit(0);
                    }


                }

                if (opt_url == "")
                {
                    //Console.WriteLine("Invalid URL, aborting...");
                    ShowHelp();
                    Environment.Exit(1);
                }

                // purge off the leading https if its there...
                opt_url = opt_url.Replace("https://", "");


                var request = WebRequest.Create("https://" + opt_url + ":" + opt_port);
                var response = request.GetResponse();

                
            }

            private static void ShowHelp()
            {
                string help = @"Usage: chkssl [-d] [-c] [-r] [-p PORT] host
Flags:
    -p : Specify tcp port, default is 443
    -a : Turns on all the extra output (except raw)
    -d : Include more details about the certificate
    -c : Include all certificates in the chain
    -r : Dump the raw certificate
    -v : Show version information
    -h : Show this help

chkssl.exe by Eli Fulkerson
Please see http://www.elifulkerson.com/projects/ for updates.
";
                Console.WriteLine(help);
            }

            private static void ShowVersion()
            {
                string version = @"chkssl.exe 0.1 Nov 19 2014

chkssl.exe by Eli Fulkerson
Please see http://www.elifulkerson.com/projects/ for updates.

";
                Console.WriteLine(version);
            }

            private static bool FormatCertificate(X509Certificate2 X2)
            {
                bool retval = false;


                //Console.WriteLine("Status      Issued                  Expires                            Left");
                Console.WriteLine("{0,-8} {1,-23} {2,-23} {3}", "Status", "Start Date", "Expires", "Days Left");
                Console.WriteLine("{0,-8} {1,-23} {2,-23} {3}", "-------", "-----------------------", "----------------------", "---------");
                if (X2.Verify())
                {
                    Console.Write("VALID    ");
                    retval = true;
                }
                else
                {
                    Console.Write("INVALID  ");
                    retval = false;
                }

                DateTime na = Convert.ToDateTime(X2.NotAfter);
                Console.Write("{0,-23} {1,-23} {2,9}", X2.NotBefore, X2.NotAfter, Convert.ToInt32((na - DateTime.Today).TotalDays));

                if (opt_showCertDetails)
                {
                    Console.WriteLine("\n");
                    Console.WriteLine("{0,-9} {1}", "Subject:", X2.SubjectName.Name);
                    Console.WriteLine("{0,-9} {1}", "Issuer:", X2.IssuerName.Name);
                    Console.WriteLine("{0,-9} {1}", "Serial:", X2.GetSerialNumberString());
                    Console.WriteLine("{0,-9} {1}", "Version:", X2.Version);
                    Console.WriteLine("{0,-9} {1}", "Format:", X2.GetFormat());
                    Console.WriteLine("{0,-9} {1}", "Hash:", X2.GetCertHashString());
                    Console.WriteLine("\n");
                    Console.WriteLine("{0,-20} {1} {2} {3}", "Key Algorithm:", X2.GetKeyAlgorithm(), "Parameters:", X2.GetKeyAlgorithmParametersString());
                    Console.WriteLine("{0,-20} {1}", "Signature Algorithm:", X2.SignatureAlgorithm.FriendlyName);
                }
                if (opt_showRaw)
                {

                    Console.WriteLine("\n");
                    Console.WriteLine("Raw Cert:");
                    Console.WriteLine("---------");
                    Console.WriteLine(X2.GetRawCertDataString());
                    Console.WriteLine("---------");
                }
                Console.WriteLine("\n");

                return retval;

            }


            private static bool ServerCertificateValidationCallback(object sender, X509Certificate certificate, X509Chain chain, SslPolicyErrors sslPolicyErrors)
            {        
                Console.WriteLine("SSL check run on: " + DateTime.Now.ToString());
                Console.WriteLine("\n");

                // some of the data I'm after is available in X509Certificate, other is available in X509Certificate2.  Ugh.
                X509Certificate2 X2 = new X509Certificate2();
                //X2.Import(certificate.GetRawCertData());

                //bool retval = FormatCertificate(X2);
                bool retval = true;
                bool meh = false;

                
                //Console.WriteLine("Certificate Chain:");
                //Console.WriteLine("------------------");
                //Console.WriteLine("\n");

                for (int i = 0; i < chain.ChainElements.Count; i++)
                {

                    X2.Import(chain.ChainElements[i].Certificate.GetRawCertData());
                    meh = FormatCertificate(X2);

                    // if we ever get a failed cert, return false
                    if (!meh) { retval = false; }

                    if (opt_showChain == false)
                    {
                        break;
                    }
                    
                }



                if (retval)
                {
                    Environment.Exit(0);
                }
                else
                {
                    Environment.Exit(1);
                }

                //hah!
                return true;
            }
        }
    
}
